package com.caimei365.manager.service.impl;

import com.caimei365.manager.dao.SysUserDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.SysMenu;
import com.caimei365.manager.entity.SysUser;
import com.caimei365.manager.service.RedisService;
import com.caimei365.manager.service.SysUserService;
import com.github.pagehelper.PageHelper;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/12/10
 */
@Service
public class SysUserServiceImpl implements SysUserService {

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private BCryptPasswordEncoder passwordEncoder;

    @Resource
    private RedisService redisService;

    /**
     * 获取用户信息
     */
    @Override
    public ResponseJson<SysUser> getInfoByUsername(String username) {
        if (StringUtils.hasLength(username)) {
            SysUser sysUser = sysUserDao.findByUsername(username);
            // 获取用户角色列表
            List<String> roleList = sysUserDao.getRoleList(sysUser.getId());
            sysUser.setRoles(roleList);

            // 获取用户角色Id
            List<Integer> roleIds = sysUserDao.getRoleIdsByUserId(username);
            List<SysMenu> menus = null;
            if (!CollectionUtils.isEmpty(roleIds)) {
                // 根据角色Id获取菜单列表
                menus = sysUserDao.getMenusByRoleIds(roleIds);
            }
            sysUser.setMenus(menus);

            return ResponseJson.success(sysUser);
        }
        return ResponseJson.error("用户数据异常", null);
    }

    /**
     * 获取菜单列表
     *
     * @param parentId 父级菜单Id
     * @param level    菜单层级
     * @param status   状态：0启用，1停用
     * @param pageNum  页码
     * @param pageSize 每页大小
     */
    @Override
    public ResponseJson<PaginationVo<SysMenu>> getMenuList(Integer parentId, Integer level, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<SysMenu> menus = sysUserDao.getMenuList(parentId, level, status);
        PaginationVo<SysMenu> pageData = new PaginationVo<>(menus);
        return ResponseJson.success(pageData);
    }

    /**
     * 根据ID获取菜单
     *
     * @param id
     */
    @Override
    public ResponseJson<SysMenu> getMenu(Integer id) {
        SysMenu menu = sysUserDao.getMenu(id);
        return ResponseJson.success(menu);
    }

    /**
     * 根据ID更新菜单
     *
     * @param id
     */
    @Override
    public ResponseJson<Void> updateMenu(Integer id, SysMenu menu) {
        menu.setId(id);
        sysUserDao.updateMenu(menu);
        return ResponseJson.success();
    }

    /**
     * 根据ID删除菜单
     *
     * @param id
     */
    @Override
    public ResponseJson<Void> deleteMenu(Integer id) {
        sysUserDao.deleteMenu(id);
        return ResponseJson.success();
    }

//    /**
//     * 获取用户可访问菜单
//     */
//    @Override
//    public ResponseJson<List<SysMenu>> menuList(String username) {
//        if (!StringUtils.hasLength(username)) {
//            return ResponseJson.error("用户信息异常", null);
//        }
//        // 获取用户角色Id
//        List<Long> roleIds = sysUserDao.getRoleIdsByUserId(username);
//        List<SysMenu> menus = null;
//        if (!CollectionUtils.isEmpty(roleIds)) {
//            // 根据角色Id获取菜单列表
//            menus = sysUserDao.getMenuListByRoleIds(roleIds);
//        }
//        return ResponseJson.success(menus);
//    }

    /**
     * 退出登录
     *
     * @param token
     */
    @Override
    public ResponseJson<Void> logout(String token) {
        if (StringUtils.hasLength(token)) {
            redisService.remove(token);
        }
        return ResponseJson.success();
    }

    @Override
    public ResponseJson<SysUser> register(SysUser sysUser) {
        if (StringUtils.hasLength(sysUser.getUsername()) && StringUtils.hasLength(sysUser.getPassword())) {
            // 密码加密
            String encodePassword = passwordEncoder.encode(sysUser.getPassword());
            sysUser.setPassword(encodePassword);
            // 新增用户
            sysUserDao.insertSysUser(sysUser);
            // 角色Ids，用","隔开
            String roleIds = sysUser.getRoleIds();
            if (StringUtils.hasLength(roleIds)) {
                // 设置用户角色
                String[] split = roleIds.split(",");
                for (String s : split) {
                    if (StringUtils.hasLength(s)) {
                        // 保存用户角色关系
                        sysUserDao.insertUserRoleRelation(sysUser.getId(), Integer.valueOf(s));
                    }
                }
            }
            return ResponseJson.success("注册成功", sysUser);
        }
        return ResponseJson.error("用户名或密码不能为空", null);
    }

}
